@@ -16,7 +16,9 @@
 #' @param zr                Instrument (reference) height (m)
 #' @param zh                Canopy height (m)
 #' @param d                 Zero-plane displacement height (m)
-#' @param z0m               Roughness length for momentum (m)
+#' @param z0m               Roughness length for momentum (m), optional; if not provided, it is estimated from \code{roughness.parameters}
+#'                          (method="wind_profile"). Only used if \code{wind_profile = TRUE} and/or \code{Rb_model} = \code{"Su_2001"} or
+#'                          \code{"Choudhury_1988"}.
 #' @param Dl                Characteristic leaf dimension (m) (if \code{Rb_model} = \code{"Su_2001"}) 
 #'                          or leaf width (if \code{Rb_model} = \code{"Choudhury_1988"}); ignored otherwise.
 #' @param N                 Number of leaf sides participating in heat exchange (1 or 2); only used if \code{Rb_model = "Su_2001"}.
@@ -33,7 +35,7 @@
 #' @param Rb_model          Boundary layer resistance formulation. One of \code{"Thom_1972","Choudhury_1988","Su_2001","constant_kB-1"}.
 #' @param kB_h              kB-1 value for heat transfer; only used if \code{Rb_model = "constant_kB-1"}
 #' @param Sc                Optional: Schmidt number of additional quantities to be calculated
-#' @param Sc_name           Optional: Name of the additonal quantities, has to be of same length than 
+#' @param Sc_name           Optional: Name of the additional quantities, has to be of same length than 
 #'                          \code{Sc_name}
 #' @param constants         k - von Karman constant \cr
 #'                          cp - specific heat of air for constant pressure (J K-1 kg-1) \cr
@@ -94,17 +96,23 @@
 #'  The option \code{Rb_model = "Su_2001"} calculates Rb_h based on the physically-based Rb model by Su et al. 2001,
 #'  a simplification of the model developed by Massman 1999:
 #'  
-#'     \deqn{kB-1 = (k Cd fc^2) / (4Ct ustar/u(zh)) + kBs-1(1 - fc)^2}
+#'     \deqn{kB_h = (k Cd fc^2) / (4Ct ustar/u(zh)) + kBs-1(1 - fc)^2}
 #'     
 #'  where Cd is a foliage drag coefficient (defaults to 0.2), fc is fractional
 #'  vegetation cover, Bs-1 is the inverse Stanton number for bare soil surface,
 #'  and Ct is a heat transfer coefficient. See \code{\link{Gb.Su}} for 
 #'  details on the model.
 #'     
+#'  The models calculate the parameter kB^(-1) (in the code referred to as \code{kB_h}), 
+#'  which is related to Rb_h:
 #'  
-#'  The models calculate the parameter kB-1, which is related to Rb_h:
+#'     \deqn{kB_h = Rb_h * (k * ustar)}
+#'     
+#'  From version 0.7.6 onwards, the roughness length for heat (z0h) is added to the output
+#'  if z0m is available (i.e. provided as input or calculated within this function).
+#'  z0h is calculated from \code{\link{roughness.length.heat}}:
 #'  
-#'     \deqn{kB-1 = Rb_h * (k * ustar)}
+#'     \deqn{z0h = z0m / exp(kB_h)}
 #'  
 #'  Rb (and Gb) for water vapor and heat are assumed to be equal in this package.
 #'  Gb for other quantities x is calculated as (Hicks et al. 1987):
@@ -113,14 +121,16 @@
 #'  
 #'  where Sc_x is the Schmidt number of quantity x, and Pr is the Prandtl number (0.71).
 #'  
-#' @return a dataframe with the following columns:
+#' @return a data.frame with the following columns:
 #'         \item{Ga_m}{Aerodynamic conductance for momentum transfer (m s-1)}
 #'         \item{Ra_m}{Aerodynamic resistance for momentum transfer (s m-1)}
 #'         \item{Ga_h}{Aerodynamic conductance for heat transfer (m s-1)}
 #'         \item{Ra_h}{Aerodynamic resistance for heat transfer (s m-1)}
 #'         \item{Gb_h}{Canopy boundary layer conductance for heat transfer (m s-1)}
 #'         \item{Rb_h}{Canopy boundary layer resistance for heat transfer (s m-1)}
-#'         \item{kB_h}{kB-1 parameter for heat transfer}
+#'         \item{kB_h}{kB^(-1) parameter for heat transfer}
+#'         \item{z0h}{Roughness length for heat (m) (NA if not input \code{z0m} not provided as input
+#'                    or not estimated in this function)}
 #'         \item{zeta}{Stability parameter 'zeta' (NA if \code{wind_profile = FALSE})}
 #'         \item{psi_h}{Integrated stability correction function (NA if \code{wind_profile = FALSE})}
 #'         \item{Ra_CO2}{Aerodynamic resistance for CO2 transfer (s m-1)}
@@ -131,23 +141,19 @@
 #'         \item{Gb_Sc_name}{Boundary layer conductance for \code{Sc_name} (m s-1). Only added if \code{Sc_name} and 
 #'                           the respective \code{Sc} are provided}
 #'        
-#' @note The roughness length for water and heat (z0h) is not returned by this function, but 
-#'       it can be calculated from the following relationship (e.g. Verma 1989):
-#'       
-#'       \deqn{kB-1 = ln(z0m/z0h)} 
-#'       
-#'       it follows:
-#'       
-#'       \deqn{z0h = z0m / exp(kB-1)}
-#'       
-#'       \code{kB-1} is an output of this function.
-#'       
-#'       Input variables such as LAI, Dl, or zh can be either constants, or
+#' @note Input variables such as LAI, Dl, or zh can be either constants, or
 #'       vary with time (i.e. vectors of the same length as \code{data}).
 #'       
 #'       Note that boundary layer conductance to water vapor transfer (Gb_w) is often 
 #'       assumed to equal Gb_h. This assumption is also made in this R package, for 
 #'       example in the function \code{\link{surface.conductance}}.
+#'       
+#'       If the roughness length for momentum (\code{z0m}) is not provided as input, it is estimated 
+#'       from the function \code{roughness.parameters} within \code{wind.profile} if \code{wind_profile = TRUE} 
+#'       and/or \code{Rb_model} = \code{"Su_2001"} or \code{"Choudhury_1988"} The \code{roughness.parameters}
+#'       function estimates a single \code{z0m} value for the entire time period! If a varying \code{z0m} value 
+#'       (e.g. across seasons or years) is required, \code{z0m} should be provided as input argument.
+#'       
 #'         
 #' @references Verma, S., 1989: Aerodynamic resistances to transfers of heat, mass and momentum.
 #'             In: Estimation of areal evapotranspiration, IAHS Pub, 177, 13-20.
@@ -173,16 +179,23 @@
 #' # calculation of Ga using a model derived from the logarithmic wind profile
 #' aerodynamic.conductance(df,Rb_model="Thom_1972",zr=40,zh=25,d=17.5,z0m=2,wind_profile=TRUE) 
 #' 
-#' # simple calculation of Ga, but a physically based canopy boundary layer model
+#' # simple calculation of Ga_m, but a physically based canopy boundary layer model
 #' aerodynamic.conductance(df,Rb_model="Su_2001",zr=40,zh=25,d=17.5,Dl=0.05,N=2,fc=0.8)
 #' 
+#' @importFrom utils packageVersion
+#' 
 #' @export
 aerodynamic.conductance <- function(data,Tair="Tair",pressure="pressure",wind="wind",ustar="ustar",H="H",
-                                    zr,zh,d,z0m,Dl,N=2,fc=NULL,LAI,Cd=0.2,hs=0.01,wind_profile=FALSE,
+                                    zr,zh,d,z0m=NULL,Dl,N=2,fc=NULL,LAI,Cd=0.2,hs=0.01,wind_profile=FALSE,
                                     stab_correction=TRUE,stab_formulation=c("Dyer_1970","Businger_1971"),
                                     Rb_model=c("Thom_1972","Choudhury_1988","Su_2001","constant_kB-1"),
                                     kB_h=NULL,Sc=NULL,Sc_name=NULL,constants=bigleaf.constants()){
   
+  pv <- packageVersion("bigleaf")
+  if (pv > "0.7.5"){
+    cat("Note new column 'z0h' in the function output for 'bigleaf' versions > 0.7.5.",fill=TRUE)
+  }
+  
   Rb_model         <- match.arg(Rb_model)
   stab_formulation <- match.arg(stab_formulation)
   
@@ -198,14 +211,14 @@ aerodynamic.conductance <- function(data,Tair="Tair",pressure="pressure",wind="w
     } else if (Rb_model == "Choudhury_1988"){
       
       Gb_mod <- Gb.Choudhury(data,Tair=Tair,pressure=pressure,wind=wind,ustar=ustar,
-                             H=H,leafwidth=Dl,LAI=LAI,zh=zh,zr=zr,d=d,
+                             H=H,leafwidth=Dl,LAI=LAI,zh=zh,zr=zr,d=d,z0m=z0m,
                              stab_formulation=stab_formulation,Sc=Sc,Sc_name=Sc_name,
                              constants=constants)
       
     } else if (Rb_model == "Su_2001"){
       
       Gb_mod <- Gb.Su(data=data,Tair=Tair,pressure=pressure,ustar=ustar,wind=wind,
-                      H=H,zh=zh,zr=zr,d=d,Dl=Dl,N=N,fc=fc,LAI=LAI,Cd=Cd,hs=hs,
+                      H=H,zh=zh,zr=zr,d=d,z0m=z0m,Dl=Dl,N=N,fc=fc,LAI=LAI,Cd=Cd,hs=hs,
                       stab_formulation=stab_formulation,Sc=Sc,Sc_name=Sc_name,
                       constants=constants)  
       
@@ -246,6 +259,16 @@ aerodynamic.conductance <- function(data,Tair="Tair",pressure="pressure",wind="w
   ## calculate aerodynamic conductance for momentum (Ga_m)
   if (wind_profile){
     
+    if (is.null(z0m) & Rb_model %in% c("constant_kB-1","Thom_1972")){
+      stop("z0m must be provided if wind_profile=TRUE!")
+    } else if (is.null(z0m) & Rb_model %in% c("Choudhury_1988","Su_2001")){
+      # z0m estimated as in Choudhury_1988 or Su_2001
+      z0m <- roughness.parameters(method="wind_profile",zh=zh,zr=zr,d=d,data=data,
+                                  Tair=Tair,pressure=pressure,wind=wind,ustar=ustar,H=H,
+                                  stab_roughness=TRUE,stab_formulation=stab_formulation,
+                                  constants=constants)[,"z0m"]
+    }
+    
     if (stab_correction){
       
       zeta  <-  stability.parameter(data=data,Tair=Tair,pressure=pressure,ustar=ustar,
@@ -287,9 +310,15 @@ aerodynamic.conductance <- function(data,Tair="Tair",pressure="pressure",wind="w
   Ra_CO2 <- 1/Ga_x[,1]
   colnames(Ga_x) <- paste0("Ga_",c("CO2",Sc_name))
   
+  if(!is.null(z0m)){
+    z0h <- roughness.length.heat(z0m,kB_h)
+  } else {
+    z0h <- rep(NA_integer_,length=length(Ra_m))
+  }
+  
   Gab_x <- cbind(Ga_x,Gb_x)
   Gab_x <- Gab_x[rep(c(1,ncol(Gab_x)-(ncol(Gab_x)/2-1)),ncol(Gab_x)/2) + sort(rep(0:(ncol(Gab_x)/2-1),2))] # reorder columns
 
-  return(data.frame(Ga_m,Ra_m,Ga_h,Ra_h,Gb_h,Rb_h,kB_h,zeta,psi_h,Ra_CO2,Gab_x))
+  return(data.frame(Ga_m,Ra_m,Ga_h,Ra_h,Gb_h,Rb_h,kB_h,z0h,zeta,psi_h,Ra_CO2,Gab_x))
   
 }
\ No newline at end of file
